import { ChangeDetectionStrategy, Component, ContentChild, EventEmitter, HostListener, Input, Output, TemplateRef } from "@angular/core";
import { SxplrRegion } from "src/atlasComponents/sapi/sxplrTypes";
import { ARIA_LABELS } from "common/constants"
import { UntypedFormControl } from "@angular/forms";
import { debounceTime, distinctUntilChanged, map, startWith } from "rxjs/operators";
import { MatAutocompleteSelectedEvent } from "@angular/material/autocomplete";
import { SapiViewsCoreRichRegionListTemplateDirective } from "./regionListSearchTmpl.directive";

const filterRegionViaSearch = (searchTerm: string) => (region:SxplrRegion) => {
  return region.name.toLocaleLowerCase().includes(searchTerm.toLocaleLowerCase())
}

@Component({
  selector: `sxplr-sapiviews-core-rich-regionlistsearch`,
  templateUrl: './regionListSearch.template.html',
  styleUrls: [
    `./regionListSearch.style.css`
  ],
  changeDetection: ChangeDetectionStrategy.OnPush
})

export class SapiViewsCoreRichRegionListSearch {

  ARIA_LABELS = ARIA_LABELS

  showNOptions = 4

  private _regions: SxplrRegion[] = []
  get regions(){
    return this._regions
  }
  @Input('sxplr-sapiviews-core-rich-regionlistsearch-regions')
  set regions(reg: SxplrRegion[]) {
    this._regions = reg.filter(r => !reg.some(c => c.parentIds.includes(r.id)))
  }

  @ContentChild(SapiViewsCoreRichRegionListTemplateDirective)
  regionTmplDirective: SapiViewsCoreRichRegionListTemplateDirective

  @Input('sxplr-sapiviews-core-rich-regionlistsearch-region-template-ref')
  regionTemplateRef: TemplateRef<any>

  @Input('sxplr-sapiviews-core-rich-regionlistsearch-current-search')
  currentSearch: string = ''

  @Output('sxplr-sapiviews-core-rich-regionlistsearch-region-select')
  onOptionSelected = new EventEmitter<SxplrRegion>()
  
  @Output('sxplr-sapiviews-core-rich-regionlistsearch-region-toggle')
  onRegionToggle = new EventEmitter<SxplrRegion>()

  public searchFormControl = new UntypedFormControl()

  public searchedList$ = this.searchFormControl.valueChanges.pipe(
    startWith(''),
    distinctUntilChanged(),
    debounceTime(160),
    map((searchTerm: string | SxplrRegion) => {
      if (typeof searchTerm === "string") {
        return this.regions.filter(filterRegionViaSearch(searchTerm))
      }
      return []
    })
  )

  public autocompleteList$ = this.searchedList$.pipe(
    map(list => list.slice(0, this.showNOptions))
  )

  displayFn(region: SxplrRegion){
    return region?.name || ''
  }

  optionSelected(opt: MatAutocompleteSelectedEvent) {
    const selectedRegion = opt.option.value as SxplrRegion
    if (this.ctrlFlag) {
      this.onRegionToggle.emit(selectedRegion)
    } else {
      this.onOptionSelected.emit(selectedRegion)
    }
  }

  ctrlFlag = false

  @HostListener('document:keydown', ['$event'])
  keydown(event: KeyboardEvent) {
    this.ctrlFlag = event.ctrlKey
  }
  
  @HostListener('document:keyup', ['$event'])
  keyup(event: KeyboardEvent) {
    this.ctrlFlag = event.ctrlKey
  }
}
